/*
 * $Id: ConfirmApplicationDeliveryReceivingServlet.java,v 1.6 2003/09/15 16:39:00 grantsuser Exp $
 * $Revision: 1.6 $
 * $Date: 2003/09/15 16:39:00 $
 */

package gov.grants.webSvcs.servlet.receiver;

// Sun
import javax.xml.soap.*;
import javax.xml.messaging.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.xml.transform.*;
import javax.naming.*;

import java.io.InputStream;

// jwsdp - sun
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
//import javax.xml.parsers.ParserConfigurationException;

// Grants.gov
import gov.grants.utility.GrantsGovConstantsUtils;
import gov.grants.utility.GrantsGovConstants;
import gov.grants.webSvcs.utility.GrantsGovWebSvcsMediator;
import gov.grants.webSvcs.utility.GrantsGovWebSvcsMediatorStub;
import gov.grants.webSvcs.utility.SOAPUtils;

/**
 * Servlet that receives a ConfirmApplicationDelivery() message, and returns
 * a confirmation message.
 *
 * Based on the sample servlet, ReceivingServlet.java, provided in
 * jaxm-simple.war
 *
 * @author Mark Sommer
 */
public class ConfirmApplicationDeliveryReceivingServlet extends JAXMServlet implements ReqRespListener {

	private static MessageFactory messageFactory = null;
	private static DocumentBuilderFactory docBuilderFactory;
	private static DocumentBuilder docBuilder;

	private static String appPath = null;
	private static String grants_govTrackingNumber = null;
    private static ServletContext context;
	private static boolean isSuccess;
	private static SOAPMessage responseMsg;
	private static GrantsGovWebSvcsMediator grantsGovWebSvcsMediator;

	static {
        try {
			GrantsGovConstantsUtils.setSystemProperties();
            messageFactory = MessageFactory.newInstance();
        } catch (Exception e) {
            e.printStackTrace();
        }
    };

    public void init(ServletConfig servletConfig) throws ServletException {
        super.init(servletConfig);

		try {
			context = getServletContext();
			System.out.println( "context=" + context );
			appPath = GrantsGovConstantsUtils.getApplicationPath( context.getRealPath( "/" ) );// application path
			System.out.println( "appPath= " + appPath );

			docBuilderFactory = DocumentBuilderFactory.newInstance();
			docBuilderFactory.setNamespaceAware(true);
			docBuilder = docBuilderFactory.newDocumentBuilder();

			grantsGovWebSvcsMediator = new GrantsGovWebSvcsMediatorStub( context );

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

	//
    // This is the application code for handling the message.
	//
    public SOAPMessage onMessage(SOAPMessage requestMessage) {
        System.out.println("On message called in receiving servlet");
        try {
            System.out.println( "Here's the request message: " );
            requestMessage.writeTo(System.out);
			System.out.println( "\n\n" );

			// Get the Grants.gov Tracking Number
			grants_govTrackingNumber = SOAPUtils.getSOAPElement( requestMessage, "Grants_govTrackingNumber", "ConfirmApplicationDelivery" );

			// Confirm application delivery
			isSuccess = grantsGovWebSvcsMediator.confirmApplicationDelivery( grants_govTrackingNumber );

			// create the response
            responseMsg = createResponseMsg( isSuccess, grants_govTrackingNumber );

			System.out.println( "The confirmation response message is ready to be returned" );
            return responseMsg;

        } catch(Exception e) {
            System.out.println("Error in processing or replying to a message:  " + e);
            return null;
        }
    }

	private SOAPMessage createResponseMsg( boolean isSuccess, String grants_govTrackingNumber )
	throws javax.xml.soap.SOAPException, org.xml.sax.SAXException, java.io.IOException {
		System.out.println( "In createResponseMsg" );
		SOAPMessage responseMsg = messageFactory.createMessage();
		SOAPEnvelope soapEnvelope = responseMsg.getSOAPPart().getEnvelope();
		SOAPBody responseBody = soapEnvelope.getBody();
		SOAPElement rootElement = responseBody.addChildElement(soapEnvelope.createName( "ConfirmApplicationDeliveryResponse", GrantsGovConstants.AGIS_PREFIX, GrantsGovConstants.AGIS_URI ));
		SOAPElement confirmationResponseMessageElement;
		SOAPElement grants_govTrackingNumberElement;

		grants_govTrackingNumberElement = rootElement.addChildElement(soapEnvelope.createName("Grants_govTrackingNumber"));
		grants_govTrackingNumberElement.addTextNode(grants_govTrackingNumber);

		String confirmMsg = "Failed";

		if ( isSuccess ) {
			confirmMsg = "Succeed";
		}

		confirmationResponseMessageElement = rootElement.addChildElement(soapEnvelope.createName("ConfirmationResponseMessage"));
		confirmationResponseMessageElement.addTextNode(confirmMsg);

		return responseMsg;

	}// createResponseMsg()
}

