/*
 * $Id: GetApplicationReceivingServlet.java,v 1.12 2003/09/15 16:38:34 grantsuser Exp $
 * $Revision: 1.12 $
 * $Date: 2003/09/15 16:38:34 $
 */

package gov.grants.webSvcs.servlet.receiver;

// Sun

import java.io.InputStream;
import java.net.URL;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import javax.activation.DataHandler;
import javax.xml.soap.*;
import javax.xml.messaging.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.naming.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;

// jwsdp - sun
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
//import javax.xml.parsers.ParserConfigurationException;

// Grants.gov
import gov.grants.utility.GrantsGovConstantsUtils;
import gov.grants.utility.GrantsGovConstants;
import gov.grants.webSvcs.utility.GrantsGovWebSvcsMediator;
import gov.grants.webSvcs.utility.GrantsGovWebSvcsMediatorStub;
import gov.grants.webSvcs.utility.SOAPUtils;
import gov.grants.webSvcs.utility.AttachmentDataSource;
//import gov.grants.webSvcs.valueobject.AttachmentInformation;

/**
 * Servlet that receives a getApplication() message, and returns
 * an application.
 *
 * Based on the sample servlet, ReceivingServlet.java, provided in
 * jaxm-simple.war
 *
 * @author Dan Dumitru
 */
public class GetApplicationReceivingServlet extends JAXMServlet implements ReqRespListener {

	private static MessageFactory messageFactory = null;
	private static DocumentBuilderFactory docBuilderFactory;
	private static DocumentBuilder docBuilder;

	private static String appPath = null;
	private static String grants_govTrackingNumber = null;
    private static ServletContext context;
	private static GrantsGovWebSvcsMediator grantsGovWebSvcsMediator;

	static {
        try {
			GrantsGovConstantsUtils.setSystemProperties();
            messageFactory = MessageFactory.newInstance();
        } catch (Exception e) {
            e.printStackTrace();
        }
    };

    public void init(ServletConfig servletConfig) throws ServletException {
        super.init(servletConfig);

		try {
			context = getServletContext();
			System.out.println( "context=" + context );
			appPath = GrantsGovConstantsUtils.getApplicationPath( context.getRealPath( "/" ) );// application path
			System.out.println( "appPath= " + appPath );

			docBuilderFactory = DocumentBuilderFactory.newInstance();
			docBuilderFactory.setNamespaceAware(true);
			docBuilder = docBuilderFactory.newDocumentBuilder();

			grantsGovWebSvcsMediator = new GrantsGovWebSvcsMediatorStub( context );

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

	//
    // This is the application code for handling the message.
	//
    public SOAPMessage onMessage(SOAPMessage requestMessage) {
        System.out.println("On message called in receiving servlet");

		SOAPMessage responseMsg;
		InputStream application;
		List attachments;

        try {
            System.out.println( "Here's the request message: " );
            requestMessage.writeTo(System.out);
			System.out.println( "\n\n" );

			try {
				// Get the Grants.gov Tracking Number
				grants_govTrackingNumber = SOAPUtils.getSOAPElement( requestMessage, "Grants_govTrackingNumber", "GetApplication" );
			} catch( Exception e ) {
				System.out.println( "Unable to retrieve GrantsGov Tracking Number:  " + e );
				responseMsg = SOAPUtils.generateSOAPFaultMsg( messageFactory,
															  "Unable to retrieve GrantsGov Tracking Number:  " + e,
															  GrantsGovConstants.LOCAL_NAME_CLIENT);
				return responseMsg;
			}

			// Get the Application
			application = grantsGovWebSvcsMediator.getApplication( grants_govTrackingNumber );

			// Get any attachments
			attachments = grantsGovWebSvcsMediator.getAttachments( grants_govTrackingNumber );

			// create the response
            responseMsg = createResponseMsg( application, attachments );

			System.out.println( "The response message is ready to be returned" );
            return responseMsg;

        } catch( Exception e ) {
            System.out.println( "Error in processing or replying to a message:  " + e );
			try {
				responseMsg = SOAPUtils.generateSOAPFaultMsg( messageFactory,
															  "Error in processing or replying to a message:  " + e.getMessage(),
															  GrantsGovConstants.LOCAL_NAME_SERVER );
				return responseMsg;
			} catch( Exception ex ) {
				System.out.println( "Error trying to generate a SOAP Fault Message:  " + ex );
				return null;
			}
        }
    }

	private SOAPMessage createResponseMsg( InputStream application, List attachments )
	throws javax.xml.soap.SOAPException, org.xml.sax.SAXException, java.io.IOException {

		// JWSDP 1.1
		//
		// Need to include SOAP Envelope and Body around Application Input Stream:
		//
		// <SOAP-ENV:Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
		//  <SOAP-ENV:Body>
		//  ...
		//  </SOAP-ENV:Body>
		// </SOAP-ENV:Envelope>

		SOAPMessage responseMsg = messageFactory.createMessage();
		responseMsg.getSOAPPart().setContent( new javax.xml.transform.stream.StreamSource( application ) ) ;
		
/*
		// JWSDP 1.1
		Document applicationDoc = docBuilder.parse( attachment );
		DOMSource domSource = new DOMSource( applicationDoc );

		SOAPPart soapPart = responseMsg.getSOAPPart();
		soapPart.setContent(domSource);
		
		// JWSDP 1.2

		Document applicationDoc = docBuilder.parse( attachment );
		DocumentFragment applicationDocFrag;
		Document responseDoc;
		Element responseMsgRoot;
		SOAPBody responseBody = responseMsg.getSOAPPart().getEnvelope().getBody();

		applicationDocFrag = applicationDoc.createDocumentFragment();
		applicationDocFrag.appendChild( applicationDoc.getDocumentElement() );
//		System.out.println( "Finished creating applicationDocFrag" );

		responseDoc = docBuilder.newDocument();
		responseMsgRoot = responseDoc.createElement( "GetApplicationResponseMessage" );
//		System.out.println( "Finished creating responseMsgElement: " + responseMsgRoot );

		Node importedNode = responseDoc.importNode( (Node)applicationDocFrag, true );
		responseMsgRoot.appendChild( importedNode );
//		System.out.println( "Finished appending applicationDoc as child to responseMsgRoot" );
		responseDoc.appendChild(responseMsgRoot);
		responseMsg.writeTo(System.out);

		responseBody.addDocument( responseDoc );
//		System.out.println( "Finished adding responseDoc to responseBody" );

		responseBody.addDocument( applicationDoc );
*/
		// Add any attachment part(s) to the message.
		if ( attachments != null && !attachments.isEmpty() ) {
			Iterator iterator = attachments.iterator();
			AttachmentDataSource attachment;
			AttachmentPart ap;
			while ( iterator.hasNext() ) {
				attachment = ( AttachmentDataSource )iterator.next();
				System.out.println( "attachment.getContentType()=" + attachment.getContentType() );
				System.out.println( "attachment.getName()=" + attachment.getName() );

				// URL url = new URL("temp");
				// ap = responseMsg.createAttachmentPart( new DataHandler(url) );
				// ap.setMimeHeader( "Content-Type", attachment.getContentType() );
				// ap.addMimeHeader("Content-Id", cId);
				// ap = responseMsg.createAttachmentPart( new DataHandler( attachment.getContent(), attachment.getContentType() ) );
				// ap = responseMsg.createAttachmentPart( attachment.getContent(), attachment.getContentType() );
				// ap.setContentType("text/html");

				ap = responseMsg.createAttachmentPart();
				ap.setDataHandler( new DataHandler( attachment ) );
				ap.setContentId( attachment.getName() );
				ap.setContentLocation( attachment.getContentLocation() );
				responseMsg.addAttachmentPart( ap );
			}
		}

		return responseMsg;

	}// createResponseMsg()
}

