package gov.grants.webSvcs.utility;

// Sun
import java.io.InputStream;
import java.util.List;

import javax.xml.soap.*;
import javax.xml.messaging.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.xml.transform.*;
import javax.naming.*;

import org.xml.sax.InputSource;

// jwsdp - sun
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

// Grants.gov
import gov.grants.utility.GrantsGovConstantsUtils;
import gov.grants.utility.GrantsGovConstants;
import gov.grants.utility.GrantApplicationProcessor;
import gov.grants.webSvcs.utility.GrantsGovWebSvcsMediator;
import gov.grants.webSvcs.utility.AttachmentDataSource;
import gov.grants.webSvcs.valueobject.*;

public class GrantsGovWebSvcsMediatorStub implements GrantsGovWebSvcsMediator {

	private static ServletContext context;

    public GrantsGovWebSvcsMediatorStub( ServletContext context ) {
		this.context = context;
	}

    /*****************
     *  PUBLIC METHODS
     ****************/

    /**
     *
     * @return possible object is
     * {@link java.lang.InputStream}
     */
    public java.io.InputStream getApplication( java.lang.String grantsGovTrackingNumber ) {
		return context.getResourceAsStream( "/WEB-INF/" + grantsGovTrackingNumber + ".xml" );
	}

	/**
     *
     * @return possible object is
     * {@link java.util.List}
     */
	public java.util.List getAttachments( java.lang.String grantsGovTrackingNumber ) {
		
		List attachments = new java.util.ArrayList();
		AttachmentDataSource attachment;
		
//		attachment = new AttachmentInformation();
//		attachment.setContent( context.getResourceAsStream( "/WEB-INF/Web Services Ref Impl Instructions_030819_vs1.0.doc" ) );
//		attachment.setContentType( "application/msword" );
		
		// Add sample PDF attachment
		attachment = new AttachmentDataSource( context.getResourceAsStream( "/WEB-INF/Data_Binding_With_JAXB.pdf" ), 
											   "application/pdf", 
											   "Data_Binding_With_JAXB.pdf",
											   "Data_Binding_With_JAXB.pdf");
		attachments.add( attachment );

		// Add sample Word attachment
		attachment = new AttachmentDataSource( context.getResourceAsStream( "/WEB-INF/Web Services Ref Impl Instructions_030819_vs1.0.doc" ), 
											   "application/msword", 
											   "Web Services Ref Impl Instructions_030819_vs1.0.doc",
											   "Web Services Ref Impl Instructions_030819_vs1.0.doc");
		attachments.add( attachment );
		
		return attachments;
	}
    /**
     *
     * @return possible object is
     * {@link java.lang.String}
	 *
	 * Input:
	 *		<xsd:complexType name="ApplicationFilterType">
	 *			<xsd:sequence>
	 *				<xsd:element name="FilterType" type="xsd:string"/>
	 *				<xsd:element name="FilterValue" type="xsd:string"/>
	 *			</xsd:sequence>
	 *		</xsd:complexType>
	 *
	 * Output:
	 *		<xsd:complexType name="ApplicationListType">
	 *			<xsd:sequence>
	 *				<xsd:element name="AvailableApplicationNumber" type="xsd:int"/>
	 *				<xsd:element name="Application" type="agis:ApplicationInformationType" minOccurs="1" maxOccurs="unbounded"/>
	 *			</xsd:sequence>
	 *		</xsd:complexType>
	 *		<xsd:complexType name="ApplicationInformationType">
	 *			<xsd:sequence>
	 *				<xsd:element ref="sys:GrantSubmissionHeader"/>
	 *				<xsd:element ref="sys:GrantSubmissionFooter"/>
	 *				<xsd:element name="Grants_GovApplicationStatus" type="agis:Grants_GovApplicationStatusType"/>
	 *			</xsd:sequence>
	 *		</xsd:complexType>GRANT00000053.xml
	 *		<xsd:complexType name="Grants_GovApplicationStatusType">
	 *			<xsd:sequence>
	 *				<xsd:element name="Status" type="xsd:string"/>
	 *				<xsd:element ref="agis:AgencyTrackingNumber"/>
	 *			</xsd:sequence>
	 *		</xsd:complexType>
     */
    public java.util.List getApplicationList( java.lang.String filterType, java.lang.String filterValue)
        throws Exception  {
			
        GrantApplicationProcessor processor = new GrantApplicationProcessor( GrantApplicationProcessor.TRANSFORM_SOURCE, 
											GrantApplicationProcessor.SOAP_CONTEXT_PATH );
        java.util.ArrayList applicationList = new java.util.ArrayList(3);
        java.lang.String grantsGovTrackingNumberArray[]={ "GRANT00000053", "GRANT00000062", "GRANT00000110" };
//        gov.grants.grantapplication.GrantApplication application;
        gov.grants.grantapplication.GrantApplicationType application;
        String grantsGovTrackingNumber;
        int size;
        String status="New";
        ApplicationInformation applicationInformation;
		java.io.InputStream inputStream;

        for (int i=0; i<3; i++){

			// Unmarshal XML file to JAXB generated java objects
			inputStream = context.getResourceAsStream( "/WEB-INF/" + grantsGovTrackingNumberArray[i] + ".xml" );

			processor.unmarshal( new javax.xml.transform.stream.StreamSource( inputStream ) );
//			application = processor.getGrantApplication();
			application = processor.getGrantApplicationType();

			size = application.getContent().size();
			System.out.println( "Number of GrantApplication components such as header or footer = " + String.valueOf( size ) );
			gov.grants.system.GrantSubmissionFooter grantSubmissionFooter = ( gov.grants.system.GrantSubmissionFooter )application.getContent().get( size - 1 );
			gov.grants.system.GrantSubmissionHeader grantSubmissionHeader = ( gov.grants.system.GrantSubmissionHeader )application.getContent().get( 0 );

			grantsGovTrackingNumber = grantSubmissionFooter.getGrantsGovTrackingNumber();
			System.out.println( "grantsGovTrackingNumber = " + grantsGovTrackingNumber );

			applicationInformation = new ApplicationInformation();
			applicationInformation.setGrantSubmissionFooter(grantSubmissionFooter);
			applicationInformation.setGrantSubmissionHeader(grantSubmissionHeader);
			applicationInformation.setStatus(status);

			applicationList.add(i,applicationInformation);

        }


//		cleanup();
		processor.resetGrantApplication();
		processor = null;

        return applicationList;
    }

	public boolean confirmApplicationDelivery( java.lang.String grantsGovTrackingNumber ) {

		System.out.println("Confirmation of Delivery for grants_govTrackingNumber " + grantsGovTrackingNumber + " succeeded");

		return true;
	}

	public boolean setAgencyTrackingNumber( java.lang.String grantsGovTrackingNumber, java.lang.String agencyTrackingNumber ) {

		System.out.println("Confirmation of Assign Agency Tracking Number for grants_govTrackingNumber " + grantsGovTrackingNumber + " was successfully set to " + agencyTrackingNumber + ".");

		return true;
	}

	private void cleanup() {

	}
}
